/*
 * Copyright (C) 2013-2015 by the University of Southern California
 * $Id$
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "global_var.h"
#include "parse.hh"
#include <getopt.h>
#include "utility.hh"
#include "version.h"

using namespace std;

//bool set_anon;
bool report_messages = false;;
enum report_options report_qry = NO_REPORT;
enum report_options report_rsp = NO_REPORT;
bool report_extra = false;
int anon_level = ANON_NONE;
bool verbose_log = false;
string command_line;

/*
	validate parse flag
*/
void valid_parse_flag(string& s)
{
	// We accept "u" to mean unified queries as a special case
	if (s.find_first_not_of("emqruQR") != string::npos) {
		cerr<<"[ERROR] -p only supports m)essages, q)ueries, r)esponses, e)rrors; capitalize QR to include packet information" << endl;
		exit(1);
	}
	if (s.find("u") !=  string::npos) {
		// xxx: backwards compabitility
		report_qry = UNIFIED_REPORT;
	}
	if (s.find("m") !=  string::npos)
		report_messages = true;
	if (s.find("m") !=  string::npos)
		report_extra = false;
	if (s.find("q") !=  string::npos)
		report_qry = NON_UNIFIED_REPORT;
	if (s.find("Q") !=  string::npos)
		report_qry = UNIFIED_REPORT;
	if (s.find("r") !=  string::npos)
		report_rsp = NON_UNIFIED_REPORT;
	if (s.find("R") !=  string::npos)
		report_rsp = UNIFIED_REPORT;
}

/*
	check anonymization level
*/
void check_anon_level(string& s)
{
	if (s == "full") {
		anon_level = ANON_FULL;
	} else if (s == "none") {
		anon_level = ANON_NONE;
	} else if (s == "srcip") {
		anon_level = ANON_SRC_IP_ONLY;
	} else {
		cerr<<"[ERROR] -a only supports 'srcip' or 'full' or 'none'."<<endl;
		exit(-1);	
	}
}

/*
	output usage
*/
void usage(const char *comm){
	cerr<<" Usage:\n "<<
		comm << " [-p mqrQR] [-a srcip|full|none] [-k KEYFILE] [-b BITS] [-i PATH] [-o PATH] [-f PREFIX] [-c CMP] [-v]\n"
		" -p mqrQR              report DNS m)essages, q)ueries, r)esponses, e)rrors,\n"
		"      			capital Q and R includes messages in those reports\n"
		"      			default is mqr (all three, non-unified)\n"
		" -a WHAT		anonymization: full (both src and dest), or srcip, \n"
		"			or none, the default)\n"
		" -k KEYFILE		specify the key file.\n"
		" -b BITS		preserve BITS high bits as un-anonymization IP;\n"
		"			default is 0, anonymizing all bits if anon is done\n"
		" -i PATH		specify input file path; required\n"
		" -o PATH		specify output path which should be directory; default\n"
		"			path is current directory\n"
		" -c COMPRESSOR		compressor, default is '/usr/bin/xz', use \"none\" to disable\n"
		" -f OUTPUT_PREFIX	prefix of the output file\n"
		"			default is timestamp of the first packet\n"
		" -v			verbose log; default is none\n"
		"\n(dnsanon version " << VERSION << ")\n";
	exit(1);
}

/*
	main function
*/
int main(int argc, char *argv[])
{
	const char *comm = argv[0];	//command
	int opt = -1;
	//int func = -1;
	int pbits = 0;
	
	string testfunc="", parse_flag = "", output_prefix = "", compressor = "/usr/bin/xz";
	string inputPath, outputPath, keyFile, tmp;
        
      	//set_anon = false;
      	command_line = "# ";
      	for (int i=0; i<argc; i++) {
      		string tmp = argv[i];
      		command_line += tmp;
      		if (i + 1 != argc)
      			command_line += " ";
      	}
      	command_line += "\n";
      	
	while ((opt = getopt(argc, argv, "a:b:c:eEf:i:k:o:p:v")) != EOF) {
		switch(opt) {
			case 'p':
				parse_flag = optarg;
                    		valid_parse_flag(parse_flag);
				break;
			case 'e':
				// xxx: backwards compatibility, depricated
				report_extra = true;
				break;
			case 'E':
				// xxx: backwards compatibility, depricated
				report_extra = false;
				break;
			case 'k':
				keyFile = optarg;
				break;
			case 'i':
				inputPath = optarg;
				break;
			case 'o':
				outputPath = optarg;
				break;
			case 'b':
                    		tmp = optarg;
                    		if (!is_number(tmp)) {
                    			cerr<<"[ERROR] -b must have a decimal number as parameter."<<endl;
                    			exit(-1);
                    		}			
				pbits = atoi(optarg);
				break;						
			case 'a':
				tmp = optarg;
				check_anon_level(tmp);
				//set_anon = true;
                    		break;
			case 'f':
				output_prefix = optarg;
                    		break;
			case 'c':
				compressor = optarg;
                    		break;
			case 'v':
				verbose_log = true;
                    		break;                    		
                    	default:
                    		usage(comm);
		}
	}
        argc--, argv++; 		//skip first
        if (argc <= 0) {		//not valid input
            usage(comm);
        }

        //check the commandline result
        
        if (parse_flag.length() == 0) {			//you must provide input
		warnx("[warn] parse option is not given, use default mqr");
		parse_flag = "mqr";
		valid_parse_flag(parse_flag);
	}
        if (inputPath.length() == 0) 			//you must provide input
		errx(1, "[error] -i PATH is required");
                
        if (outputPath.length() == 0) {
		warnx("[warn] no output path is given, use current directory");
		outputPath = "./";
	}
	
        if (anon_level != ANON_NONE && (keyFile.length() == 0))	//if -anon, it must have -k
		errx(1, "[error] -k KEYFILE is required for anonymization");
        
        if (anon_level == ANON_NONE && (keyFile.length() != 0))	//if -k without -anon, -k is ignored
		warnx("[warn] no anon flag provided, key is ignored");
        
        if (pbits < 0)
		errx(1, "[error] bits must be > 0");
        
        //call parse function
	vector<string> tmp_vec;
	str_split(compressor, tmp_vec);
	if (tmp_vec.size() < 1)
		errx(1, "[error] compressor is invalid");
	parse(inputPath, outputPath, keyFile, pbits, output_prefix, tmp_vec);
        
	return 0;
}

